/**********************************************************************/
/*                           E M M C                                  */
/*--------------------------------------------------------------------*/
/* Tarea          : contiene algunas funciones para acceder a la      */
/*                  memoria EMS (Expanded Memory)                     */
/*--------------------------------------------------------------------*/
/* Autor          : MICHAEL TISCHER                                   */
/* desarrollado el: 30.08.1988                                        */
/* ltimo update  : 29.01.1995                                        */
/*--------------------------------------------------------------------*/
/* modelo de memoria: alguno con puntero FAR a los datos, es          */
/* decir Compact, Large o Huge                                        */
/**********************************************************************/

/*== enlazar archivos Include  =======================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include "EMMC.h"

/*== Typedefs ========================================================*/

typedef unsigned char BYTE;                  /*nos construimos un byte*/
typedef unsigned int WORD;
typedef BYTE BOOL;                            /*como BOOLEAN en Pascal*/

/*== Macros ==========================================================*/

/*-- MK_FP crea de una direccin de segmento y Offset un -------------*/
/*-- puntero FAR a un objeto                                   -------*/

#ifdef MK_FP                                   /*ya se defini MK_FP?*/
 #undef MK_FP
#endif
#define MK_FP(seg, ofs) ((void far *) ((unsigned long) (seg)<<16|(ofs)))

/*-- PAGE_ADR devuelve un puntero a la pgina fsica X dentro del ----*/
/*-- Page-Frame de la memoria EMS                                 ----*/

#define PAGE_ADR(x) ((void *) MK_FP(ems_frame_seg() + ((x) << 10), 0))

/*== Constantes ======================================================*/

#define TRUE  1                  /*Constantes para el trabajo con BOOL*/
#define FALSE 0

#define EMS_INT 0x67    /*nmero de interrupcin para acceder a la EMM*/
#define EMS_ERR -1                      /*se devuelve en caso de error*/

/*== variables globales===============================================*/

BYTE emm_ec;                /*aqu se guardan los cdigos de error EMM*/

/***********************************************************************/
/*  Funcin          : E M S _ I N S T                                  */
/**--------------------------------------------------------------------**/
/*  Tarea            : Determina si hay instalada memoria EMS, y un     */
/*                     driver EMS (EMM) asociado.                       */
/*  Parmetro entrada: ninguno                                          */
/*  Valor de retorno : TRUE, si hay memoria EMS instalada, sino         */
/*                     FALSE.                                           */
/************************************************************************/

BOOL ems_inst()
 {
  static char emm_name[] = { 'E', 'M', 'M', 'X', 'X', 'X', 'X', '0' };
  union REGS regs;          /*reg. de proc. para la llamada de interr.*/
  struct SREGS sregs;       /*reg. de segm. para la llamada de interr.*/

  /*-- Construir puntero a nombre en cabecera de un contr. de disp. --*/

  regs.x.ax = 0x3567;          /*n de func.: obtener vector int. 0x67*/
  intdosx(&regs, &regs, &sregs);      /*llamar la interr. del DOS 0x21*/
  return !memcmp( MK_FP(sregs.es, 10), emm_name, sizeof emm_name );
 }

/************************************************************************/
/*  Funcin          : E M S _ N U M _ P A G E                          */
/**--------------------------------------------------------------------**/
/*  Tarea            : Obtiene el n total de pginas EMS.              */
/*  Parmetro entrada: ninguno                                          */
/*  Valor de retorno : EMS_ERR en caso de error, sino el nmero de      */
/*                     pginas EMS.                                     */
/************************************************************************/
int ems_num_page()
 {
  union REGS regs;          /*reg. de proc. para la llamada de interr.*/

  regs.h.ah = 0x42;     /*nmero de funcin: obtener nmero de pginas*/
  int86(EMS_INT, &regs, &regs);                           /*llamar EMM*/
  if ((int) (emm_ec = regs.h.ah))            /*ha aparecido un error?*/
	return(EMS_ERR);                                /*S, mostrar error*/
  else                                                  /*no hay error*/
	return( regs.x.dx );             /*devolver nmero total de pginas*/
 }

/**********************************************************************/
/*  Funcin          : E M S _ F R E E _ P A G E                        */
/**--------------------------------------------------------------------**/
/*  Tarea            : Obtiene la cantidad de pginas EMS libres.       */
/*  Parmetro entrada: ninguno                                          */
/*  Valor de retorno : EMS_ERR en caso de error, sino el nmero de      */
/*                     pginas EMS libres.                              */
/***********************************************************************/

int ems_free_page()
 {
  union REGS regs;          /*reg. de proc. para la llamada de interr.*/

  regs.h.ah = 0x42;     /*nmero de funcin: obtener nmero de pginas*/
  int86(EMS_INT, &regs, &regs);                           /*llamar EMM*/
  if ((int) (emm_ec = regs.h.ah))            /*ha aparecido un error?*/
	return(EMS_ERR);                                /*S, mostrar error*/
  else                                                  /*no hay error*/
	return( regs.x.bx );            /*devolver nmero de pginas libres*/
 }

/**********************************************************************/
/*  Funcin          : E M S _ F R A M E _ S E G                        */
/**--------------------------------------------------------------------**/
/*  Tarea            : Obtiene la dir. de segmento del EMS-Page-Frame   */
/*  Parmetro entrada: ninguno                                          */
/*  Valor de retorno : EMS_ERR en caso de error, sino la direccin de   */
/*                     segmento del Page-Frame.                         */
/***********************************************************************/

WORD ems_frame_seg()
 {
  union REGS regs;          /*reg. de proc. para la llamada de interr.*/

  regs.h.ah = 0x41;          /*n de funcin: obt.dir.segm. Page-Frame*/
  int86(EMS_INT, &regs, &regs);                           /*llamar EMM*/
  if ((int) (emm_ec = regs.h.ah))            /*ha aparecido un error?*/
	return(EMS_ERR);                                /*S, mostrar error*/
  else                                                  /*no hay error*/
	return( regs.x.bx );               /*devolver direccin de segmento*/
 }

/**********************************************************************/
/*  Funcin          : E M S _ A L L O C                                */
/**--------------------------------------------------------------------**/
/*  Tarea            : Aloja el nmero de pginas indicado y devuelve   */
/*                     un Handle para acceder a estas pginas.          */
/*                                                                      */
/*  Parmetro entrada: PAGES : el nmero de pginas a alojar            */
/*                             (de 16 KBytes cada uno)                  */
/*  Valor de retorno : EMS_ERR en caso de error, sino el Handle EMS.    */
/***********************************************************************/

int ems_alloc(int pages)
 {
  union REGS regs;          /*reg. de proc. para la llamada de interr.*/

  regs.h.ah = 0x43;                 /*nmero de funcin: aljar pginas*/
  regs.x.bx = pages;                /*fijar nmero de pginas a alojar*/
  int86(EMS_INT, &regs, &regs);                           /*llamar EMM*/
  if ((int) (emm_ec = regs.h.ah))            /*ha aparecido un error?*/
	return(EMS_ERR);                                /*S, mostrar error*/
  else                                                  /*no hay error*/
	return( regs.x.dx );                          /*devolver Handle-EMS*/
 }

/**********************************************************************/
/*  Funcin          : E M S _ M A P                                    */
/**--------------------------------------------------------------------**/
/*  Tarea            : Proyecta una de las pginas alojadas bajo        */
/*                     el Handle indicado, a un pgina fsica del       */
/*                     del Page-Frame.                                  */
/*  Parmetro entrada: HANDLE: el Handle devuelto por EMS_ALLOC         */
/*                     LOGP  : la pgina lgica (0 a n-1)               */
/*                     PHYSP : la pgina fsica (0 a 3)                 */
/*  Valor de retorno : FALSE en caso de error, sino  TRUE.              */
/***********************************************************************/

BOOL ems_map(int handle, int logp, BYTE physp)
 {
  union REGS regs;          /*reg. de proc. para la llamada de interr.*/

  regs.h.ah = 0x44;                  /*nmero de funcin: fijar Maping*/
  regs.h.al = physp;                             /*fijar pgina fsica*/
  regs.x.bx = logp;                              /*fijar pgina lgica*/
  regs.x.dx = handle;                               /*fijar Handle EMS*/
  int86(EMS_INT, &regs, &regs);                           /*llamar EMM*/
  return (!(emm_ec = regs.h.ah));
 }

/**********************************************************************/
/*  Funcin          : E M S _ F R E E                                  */
/**--------------------------------------------------------------------**/
/*  Tarea            : Devuelve la memoria alojada bajo un Handle       */
/*                     de nuevo.                                        */
/*  Parmetro entrada: HANDLE: el Handle devuelto por EMS_ALLOC         */
/*  Valor de retorno : FALSE en caso de error, sino  TRUE.              */
/***********************************************************************/
BOOL ems_free(int handle)
 {
  union REGS regs;          /*reg. de proc. para la llamada de interr.*/

  regs.h.ah = 0x45;               /*nmero de funcin: liberar pginas*/
  regs.x.dx = handle;                               /*fijar Handle EMS*/
  int86(EMS_INT, &regs, &regs);                           /*llamar EMM*/
  return (!(emm_ec = regs.h.ah));     /*si AH contiene 0, todo est ok*/
 }

/**********************************************************************/
/*  Funcin          : E M S _ V E R S I O N                            */
/**--------------------------------------------------------------------**/
/*  Tarea            : Obtiene el nmero de versin EMM.                */
/*  Parmetro entrada: ninguno                                          */
/*  Valor de retorno : EMS_ERR en caso de error, sino el nmero de      */
/*                     versin EMM.                                     */
/*  Info             : En el nmero de versin, 10 est por 1.0, 11     */
/*                     por 1.1, 34 por 3.4, etc.                        */
/***********************************************************************/

BYTE ems_version()
 {
  union REGS regs;          /*reg. de proc. para la llamada de interr.*/

  regs.h.ah = 0x46;           /*nmero de funcin: obtener versin EMS*/
  int86(EMS_INT, &regs, &regs);                           /*llamar EMM*/
  if ((int) (emm_ec = regs.h.ah))            /*ha aparecido un error?*/
	return(EMS_ERR);                                /*S, mostrar error*/
  else             /*sin error, calcula nmero de versin de cifra BCD*/
	return( (regs.h.al & 15) + (regs.h.al >> 4) * 10);
 }

/**********************************************************************/
/*  Funcin          : E M S _ S A V E _ M A P                          */
/**--------------------------------------------------------------------**/
/*  Tarea            : Guarda la proyeccin (Mapping) entre pginas     */
/*                     lgicas y fsicas.                               */
/*  Parmetro entrada: HANDLE: el Handle devuelto por EMS_ALLOC         */
/*  Valor de retorno : FALSE en caso de error, sino  TRUE.              */
/***********************************************************************/

BOOL ems_save_map(int handle)
 {
  union REGS regs;          /*reg. de proc. para la llamada de interr.*/

  regs.h.ah = 0x47;               /*nmero de funcin: guardar Mapping*/
  regs.x.dx = handle;                               /*fijar Handle EMS*/
  int86(EMS_INT, &regs, &regs);                           /*llamar EMM*/
  return (!(emm_ec = regs.h.ah));     /*si AH contiene 0, todo est ok*/
 }

/**********************************************************************/
/*  Funcin          : E M S _ R E S T O R E _ M A P                    */
/**--------------------------------------------------------------------**/
/*  Tarea            : Recupera una proyeccin entre pginas lgicas    */
/*                     y fsicas guardada anteriormente mediante        */
/*                     EMS_SAVE_MAP.                                    */
/*  Parmetro entrada: HANDLE: el Handle devuelto por EMS_ALLOC         */
/*  Valor de retorno : FALSE en caso de error, sino  TRUE.              */
/***********************************************************************/

BOOL ems_restore_map(int handle)
 {
  union REGS regs;          /*reg. de proc. para la llamada de interr.*/

  regs.h.ah = 0x48;             /*nmero de funcin: Recuperar Mapping*/
  regs.x.dx = handle;                               /*fijar Handle EMS*/
  int86(EMS_INT, &regs, &regs);                           /*llamar EMM*/
  return (!(emm_ec = regs.h.ah));     /*si AH contiene 0, todo est ok*/
 }

/**********************************************************************/
/*  Funcin          : P R I N T _ E R R                                */
/**--------------------------------------------------------------------**/
/*  Tarea            : Visualiza un mensaje de error EMS en la pantalla */
/*                     y termina el programa.                           */
/*  Parmetro entrada: ninguno                                          */
/*  Valor de retorno : ninguno                                          */
/*  Info             : Esta funcin slo se puede llamar si durante     */
/*                     la llamada anterior de una funcin del EMM       */
/*                     ha aparecido un error.                           */
/***********************************************************************/

void print_err()
 {
  static char nid[] = "no identificable";
  static char *err_vec[] =
	{  "Error en el driver EMS (EMM destruido)",                 /*0x80*/
		"Error en el hardware EMS",                               /*0x81*/
		nid,                                                      /*0x82*/
		"Handle EMM no vlido",                                   /*0x83*/
		"funcin EMS llamada no existe",                          /*0x84*/
		"no quedan Handles EMS disponibles",                      /*0x85*/
		"Error al guardar o recuperar el Mapping",                /*0x86*/
		"ms pginas pedidas de las que existen fsicamente",     /*0x87*/
		"ms pginas pedidas de las que quedan libres",           /*0x88*/
		"pedidad cero pginas",                                   /*0x89*/
		"pgina lgica no pertenece al Handle",                   /*0x8A*/
		"nmero de pgina fsico no vlido",                      /*0x8B*/
		"Zona de memoria mapping est llena",                     /*0x8C*/
		"Guardado del mapping ya se realiz",                     /*0x8D*/
		"Recuperar el mapping sin guardarlo previamente"
	};

  printf("\nATENCION! Error al acceder a la memoria EMS!r\n");
  printf("         ... %s\n", (emm_ec<0x80 || emm_ec>0x8E) ?
										 nid : err_vec[emm_ec-0x80]);
  exit( 1 );                   /*terminar programa con cdigo de error*/
 }

/**********************************************************************/
/*  Funcin          : V R _ A D R                                    */
/**------------------------------------------------------------------**/
/*  Tarea            : Devuelve un puntero a la RAM de vdeo.         */
/*  Parmetro entrada: ninguno                                        */
/*  Valor de retorno : Puntero VOID a la RAM de vdeo.                */
/**********************************************************************/

void *vr_adr()
 {
  union REGS regs;          /*reg. de proc. para la llamada de interr.*/

  regs.h.ah = 0x0f;          /*nmero de funcin:obtener modo de vdeo*/
  int86(0x10, &regs, &regs); /*llamar interrupcin de la BIOS de vdeo*/
  return ( MK_FP((regs.h.al==7) ? 0xb000 : 0xb800, 0) );
 }


/**********************************************************************/
/**                        PROGRAMA PRINCIPAL                        **/
/**********************************************************************/
////
////void main()
////{
//// int  pagenum,                                 /*nmero de pginas EMS*/
////		handle,                   /*Handle para acceder a la memoria EMS*/
////		i;                                           /*contador de bucle*/
//// WORD pageseg ;                 /*direccin de segmento del Page-Frame*/
//// BYTE emmver;                            /*nmero de versin de la EMM*/
////
//// printf("EMMC  -  (c) 1988, 92 by MICHAEL TISCHER\n\n");
//// if ( ems_inst() )                   /*Est instalada la memoria EMS?*/
////  {                                                               /*Si*/
////	/*-- Visualizar informaciones sobre la memoria EMS ----------------*/
////
////	if ( (int) (emmver = ems_version()) == EMS_ERR) /*Obten. n versin*/
////	 print_err();    /*Error: visualizar men. error y term. el programa*/
////	else                                                 /*no hay error*/
////	 printf("Nmero de versin EMM        : %d.%d\n",
////			  emmver/10, emmver%10);
////
////	if ( (pagenum = ems_num_page()) == EMS_ERR)   /*obten. n  pginas*/
////	 print_err();   /*Error: vis. men. de error y terminar el programa*/
////	printf("Nmero de pginas EMS        : %d (%d KByte)\n",
////			  pagenum, pagenum << 4);
////
////	if ( (pagenum = ems_free_page()) == EMS_ERR)
////	 print_err();      /*Error: vis. men. error y terminar el programa*/
////	printf("... libres                   : %d (%d KByte)\n",
////			 pagenum, pagenum << 4);
////
////	if ( (int) (pageseg = ems_frame_seg()) == EMS_ERR)
////	 print_err();      /*Error: vis. men. error y terminar el programa*/
////	printf("Direcc. segmento del Page-Frame: %X\n", pageseg);
////
////	printf("\nAhora se aloja una pgina de la memoria EMS, y el con-\n");
////	printf("tenido de la pantalla de la RAM de vdeo se copia a\n");
////	printf("esta pgina.\n");
////	printf("                    ... por favor pulse una tecla\n");
////	getch();                                        /*esperar una tecla*/
////
////	/*-- alojar una pgina y poyectar sobre la primera pgina lgica --*/
////	/*-- en el Page-Frame                                           ---*/
////
////	if ( (handle = ems_alloc(1)) == EMS_ERR)
////	 print_err();       /*Error: vis. men. error y terminar el programa*/
////	if ( !ems_map(handle, 0, 0) )                       /*fijar Mapping*/
////	 print_err();    /*Error: vis. men. de error y terminar el programa*/
////
////	/*-- Copiar 4000 bytes de la RAM de vdeo a la memoria EMS     ----*/
////
////	memcpy(PAGE_ADR(0), vr_adr(), 4000);
////
////	for (i=0; i<24; ++i)                           /*borrar la pantalla*/
////	 printf("\n");
////
////	printf("El antiguo contenido de la pantalla ha sido borrado y\n");
////	printf("con ello se ha perdido. Pero como ha sido guardado en\n");
////	printf("la memoria EMS, se puede volver a copiar de all a la\n");
////	printf("RAM de vdeo.\n");
////	printf("                        ... por favor pulse una tecla\n");
////	getch();                                        /*esperar una tecla*/
////
////	/*-- copiar de nuevo el contenido de la RAM de vdeo de la memo----*/
////	/*-- ria EMS y devolver la memoria EMS alojada                 ----*/
////
////	memcpy(vr_adr(), PAGE_ADR(0), 4000);    /*copiar la V-RAM de vuelta*/
////	if ( !ems_free(handle) )                          /*liberar memoria*/
////	 print_err();       /*Error: vis. men. error y terminar el programa*/
////	printf("FIN");
////  }
//// else                                  /*el driver EMS no se descubri*/
////  printf("ATENCION: No hay memoria EMS instalada.\n");
////}
